Cellular = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceDB-2.0", "AceHook-2.1", "AceConsole-2.0")

local _G = getfenv(0)
local spamname, spamtext, spamtime = "", "", 0

function Cellular:OnInitialize()
	self:RegisterDB("CellularDB")
	self:RegisterDefaults("profile", {
		height = 200,
		width = 350,
		pos = { },
		bg = "Tooltip",
		bgr = 0,
		bgg = 0,
		bgb = 0,
		border = true,
		bordr = 0.7,
		bordg = 0.7,
		bordb = 0.7,
		alpha = 0.9,
		fontsize = 12,
		inr = 1,
		ing = 0,
		inb = 1,
		outr = 0,
		outg = 1,
		outb = 1,
		busymessage = "Sorry, I'm busy right now...I'll chat with you later.",
		history = true,
		maxwindows = 8,
		fade = true,
	} )
	self:RegisterChatCommand({ "/cell", "/cellular", }, {
		type = "execute", name = "Cellular", desc = "Cellular",
		func = function() 
			self:LoadOptions() 
			self.dewdrop:Open( UIParent, 
				'children', function() self.dewdrop:FeedAceOptionsTable(self.opts) end, 
				'cursorX', true, 'cursorY', true )  
		end,
	})
	self.surface = AceLibrary("Surface-1.0")
	self.surface:Register("Tooltip", "Interface\\Tooltips\\UI-Tooltip-Background")

	self.numwindows = 0			-- number of windows created
	self.windows = { -- all windows created
		--windowId = frame,
	}			
	self.usedwindows = { -- windows assigned to whisperers
		--name = windowId,
	}		
	self.attached = nil	-- windowId where the chat editbox currently is
	self.lastwindow = nil		-- windowId of last whisper received (for reply functions)
	-- buttons layout
	self.buttons = {
		Who			= { p="TOPLEFT", x=6, y=-24, tt=1, path="Interface\\Icons\\INV_Misc_QuestionMark", func=self.Who, },
		Busy			= { p="TOPLEFT", x=6, y=-46, tt=1, path="Interface\\Icons\\Spell_Holy_Silence", func=self.Busy, },
		Invite		= { p="TOPLEFT", x=6, y=-63, tt=2, path="Interface\\Icons\\Spell_Holy_PrayerOfMendingtga", func=self.Invite, },
		Friend		= { p="TOPLEFT", x=6, y=-80, tt=2, path="Interface\\Icons\\Spell_ChargePositive", func=self.Friend, },
		Ignore		= { p="TOPLEFT", x=6, y=-100, tt=2, path="Interface\\Icons\\Spell_ChargeNegative", func=self.Ignore, },
		scrollup		= { p="BOTTOMRIGHT", x=-6, y=42, path="Interface\\ChatFrame\\UI-ChatIcon-ScrollUp-Up", func=self.ScrollUp, },
		scrolldown	= { p="BOTTOMRIGHT", x=-6, y=26, path="Interface\\ChatFrame\\UI-ChatIcon-ScrollDown-Up", func=self.ScrollDown, },
		scrollbottom	= { p="BOTTOMRIGHT", x=-6, y=10, path="Interface\\ChatFrame\\UI-ChatIcon-ScrollEnd-Up", func=self.ScrollBottom, },
		close		= { p="TOPRIGHT", x=-6, y=-6, tpl="UIPanelButtonTemplate", func=self.CloseWindow, },
		minimize		= { p="TOPRIGHT", x=-22, y=-6, tpl="UIPanelButtonTemplate", func=self.MinimizeWindow, },
	}
	
	self:RegisterEvent("CHAT_MSG_WHISPER")
	self:RegisterEvent("CHAT_MSG_WHISPER_INFORM") 
	self:RegisterEvent("CHAT_MSG_AFK") 
	self:RegisterEvent("CHAT_MSG_DND") 
	self:RegisterEvent("CHAT_MSG_IGNORED") 
	
	self:Hook("ChatFrame_MessageEventHandler", true)
	self:Hook("ChatFrame_ReplyTell", true)
	self:SecureHook(ChatFrame1, "AddMessage")
	
	-- clever way of reusing the default chat editbox
	self:HookScript(ChatFrameEditBox, "OnHide", function(object)
		self.hooks[object].OnHide()
		self:ResetEditBox()
	end)
	
	if not Cellular_History then
		Cellular_History = { }
	end
end

function Cellular:CHAT_MSG_WHISPER()
	self:IncomingMessage(arg2, arg1, arg6)
end
function Cellular:CHAT_MSG_WHISPER_INFORM()
	self:OutgoingMessage(arg2, arg1)
end
function Cellular:CHAT_MSG_AFK()
	self:IncomingMessage(arg2, "is AFK: "..arg1, nil, 2)
end
function Cellular:CHAT_MSG_DND()
	self:IncomingMessage(arg2, "is DND: "..arg1, nil, 3)
end
function Cellular:CHAT_MSG_IGNORED()
	self:IncomingMessage(arg2, "is ignoring you.", nil, 4)
end

-- don't display some messages to the chat frame
function Cellular:ChatFrame_MessageEventHandler(event)
	if event == "CHAT_MSG_WHISPER" or event == "CHAT_MSG_WHISPER_INFORM" then
		if strsub(arg1, 1, 4) == "LVPN" or strsub(arg1, 1, 4) == "LVBM" or strsub(arg1, 1, 1) == "/" then
			self.hooks.ChatFrame_MessageEventHandler(event)
		end
		return
	elseif event == "CHAT_MSG_AFK" or event == "CHAT_MSG_DND" or event == "CHAT_MSG_IGNORED" then
		return
	end
	
	self.hooks.ChatFrame_MessageEventHandler(event)
end

-- attach chat editbox to last whisper if visible
function Cellular:ChatFrame_ReplyTell(chatframe)
	local id = self.lastwindow
	if id and self.windows[id]:IsVisible() then
		self:AttachEditBox(self.windows[id].name, id)
	else
		self.hooks.ChatFrame_ReplyTell(chatframe)
	end
end

-- parse some important notifications when chatting to someone
function Cellular:AddMessage(object, text, ...)
	if event ~= "CHAT_MSG_SYSTEM" or self.numwindows == 0 then return end
	
	-- offline
	if string.find(text, "^(.+) has gone offline.") then
		local _,_,name = string.find(text, "^(.+) has gone offline.")
		self:IncomingMessage(name, "has gone offline.", nil, 1)
	-- online	
	elseif string.find(text, "|Hplayer:(.+)|h%[(.+)%]|h has come online.") then
		local _,_,name = string.find(text, "|Hplayer:(.+)|h%[(.+)%]|h has come online.")
		self:IncomingMessage(name, "has come online.", nil, 1)
	-- declines group	
	elseif string.find(text, "^(.+) declines your group invitation.") then
		local _,_,name = string.find(text, "^(.+) declines your group invitation.")
		self:IncomingMessage(name, "declines your group invitation.", nil, 1)
	end
end

-- attach the main chat editbox to the current person you're whispering
function Cellular:AttachEditBox(name, id)
	if not self.attached then
		self.l_p, self.l_rt, self.l_rp, self.l_x, self.l_y = ChatFrameEditBox:GetPoint(1)
		self.r_p, self.r_rt, self.r_rp, self.r_x, self.r_y = ChatFrameEditBox:GetPoint(2)
	end
	ChatFrameEditBox:ClearAllPoints()
	ChatFrameEditBox:SetPoint("TOPLEFT", self.windows[id], "BOTTOMLEFT", 0, 6)
	ChatFrameEditBox:SetPoint("TOPRIGHT", self.windows[id], "BOTTOMRIGHT", 0, 6)
	ChatFrame_OpenChat(string.format("/w %s ", name))
	self.attached = id
end

-- reset the editbox to it's normal position
function Cellular:ResetEditBox()
	if not self.attached then return end
	ChatEdit_OnEscapePressed(ChatFrameEditBox)
	ChatFrameEditBox:ClearAllPoints()
	if self.l_p then
		ChatFrameEditBox:SetPoint(self.l_p, self.l_rt, self.l_rp, self.l_x, self.l_y)
	end
	if self.r_p then
		ChatFrameEditBox:SetPoint(self.r_p, self.r_rt, self.r_rp, self.r_x, self.r_y)
	end
	self.attached = nil
end

-- handles all incoming whisper messages
function Cellular:IncomingMessage(name, text, status, special)
	-- filter some annoying addon whisper messages
	if strsub(text, 1, 4) == "LVPN" or strsub(text, 1, 4) == "LVBM" or strsub(text, 1, 1) == "/" then
		return
	end	
	-- cheap spam protection
	if spamname ~= "" and spamname ~= name and spamtext == text and 
	   not strfind(strlower(text), "invite") and GetTime() - spamtime < 1 then
		spamtime = GetTime()
		spamname = name
		spamtext = text
		return
	else
		spamtime = GetTime()
		spamname = name
		spamtext = text
	end
	
	-- open/create a new window if necessary
	local id = self.usedwindows[name]
	if not id or not self.windows[id]:IsVisible() then
		if special then -- don't open a new window for special messages
			return
		else
			id = self:SetupWindow(name)
		end
	end

	if id then
		self.lastwindow = id
		local f = self.windows[id]
		local tname = name
		if string.find(name, "-") then
			tname = string.sub(name, 1, string.find(name,"-") - 1) .. " (*)"
		end
		
		-- handle special messages (system, afk, dnd, etc)
		if special then
			if special == 1 then
				f.msg:AddMessage(string.format("[%s] %s %s", date("%H:%M:%S"), name, text), 1, 1, 0)
			else
				if not f.lastspecial or GetTime() > f.lastspecial + 90 or f.tag ~= special then
					f.lastspecial = GetTime()
					f.tag = special
					f.msg:AddMessage(string.format("[%s] %s %s", date("%H:%M:%S"), name, text), 1, 0, 0)
				end
			end
			return
		end
		
		-- set the last tell target so that you can /reply
		ChatEdit_SetLastTellTarget(name)
		-- show afk/dnd/gm tags
		if status and status ~= "" then
			status = string.format(" (%s)", status)
		else
			status = ""
		end
		local db = self.db.profile
		
		text = string.format(" %s", text)
		if Prat_UrlCopy and Prat:IsModuleActive(Prat_UrlCopy) then
			text = self:Decompose(text)
		end
		-- finally add the message to whisper window
		f.msg:AddMessage(string.format("[%s][%s]%s", date("%H:%M:%S"), tname..status, text), db.inr, db.ing, db.inb)
		
		-- if the whisper window is minimized, show how many whispers came during this
		if f.mini then
			f.mininew = f.mininew + 1
			f.nametext:SetText(string.format("%s (%d)", f.name, f.mininew))
		end
		
		-- add entry to history
		if db.history then
			Cellular_History[name] = Cellular_History[name] or {}
			local entry = table.getn(Cellular_History[name]) + 1
			Cellular_History[name][entry] = string.format("<%s>[%s]%s", date("%m-%d-%y %H:%M:%S"), tname, text)
		end
	else
		self:Busy(name)
	end
end

-- handles your outgoing messages
function Cellular:OutgoingMessage(name, text)
	if strsub(text, 1, 4) == "LVPN" or strsub(text, 1, 4) == "LVBM" or strsub(text, 1, 1) == "/" then
		return
	end
	-- basically same as above, just from you
	local id = self.usedwindows[name]
	if not id or not self.windows[id]:IsVisible() then
		id = self:SetupWindow(name)
	end

	if id then
		self.lastwindow = id
		local f = self.windows[id]
		local you = UnitName("player")
		local db = self.db.profile
		
		text = string.format(" %s", text)
		if Prat_UrlCopy and Prat:IsModuleActive(Prat_UrlCopy) then
			text = self:Decompose(text)
		end
		f.msg:AddMessage(string.format("[%s][%s]%s", date("%H:%M:%S"), you, text), db.outr, db.outg, db.outb)
		if db.history then
			Cellular_History[name] = Cellular_History[name] or {}
			Cellular_History[name][table.getn(Cellular_History[name]) + 1] = string.format("<%s>[%s]%s", date("%m-%d-%y %H:%M:%S"), you, text)
		end
	end
end

-- setup a new or unused window for a whisperer
function Cellular:SetupWindow(name, showmore)
	local f
	-- assumes if a window is hidden, it's unused
	for index, frame in ipairs(self.windows) do
		if not frame:IsVisible() then
			f = frame
			break
		end
	end
	
	-- cleanup
	for n,id in pairs(self.usedwindows) do
		if not self.windows[id] or not self.windows[id]:IsVisible() then
			self.usedwindows[n] = nil
		end
	end
	
	-- if no windows are available, make a new one
	if not f then 
		f = self:CreateWindow() 
	end
	-- still no windows, you probably hit maximum allowed
	if not f then return end
	
	-- setup
	f.nametext:SetText(name)
	f.msg:Clear()
	f.msg:ScrollToBottom()
	f.name = name
	f.mininew = 0
	f.lastspecial = nil
	f.tag = nil
	
	-- show recent history if available
	local history = Cellular_History[name]
	if history then
		local n = table.getn(history)
		local numlines = (showmore and 80) or 20
		for i = math.max(1, n-numlines), n, 1 do
			f.msg:AddMessage(history[i], 0.6, 0.6, 0.6)
		end
	end
	f:Show()
	self.usedwindows[name] = f.id
	
	return f.id
end

-- url patterns copy & paste from old Prat version, temp solution til I figure out current version
function Cellular:Link(link)
	return " |cffffffff|Hurl:"..link.."|h["..link.."]|h|r "
end

function Cellular:Decompose(text)
    if text then
        text = string.gsub(text, " www%.([_A-Za-z0-9-]+)%.(%S+)%s?", Cellular:Link("www.%1.%2"))
        text = string.gsub(text, " (%a+)://(%S+)%s?", Cellular:Link("%1://%2"))
        text = string.gsub(text, " ([_A-Za-z0-9-%.]+)@([_A-Za-z0-9-]+)%.([_A-Za-z0-9-%.]+)%s?", Cellular:Link("%1@%2.%3"))
        text = string.gsub(text, " (%d%d?%d?)%.(%d%d?%d?)%.(%d%d?%d?)%.(%d%d?%d?):(%d%d?%d?%d?%d?)%s?", Cellular:Link("%1.%2.%3.%4:%5"))
        text = string.gsub(text, " (%d%d?%d?)%.(%d%d?%d?)%.(%d%d?%d?)%.(%d%d?%d?)%s?", Cellular:Link("%1.%2.%3.%4"))
        text = string.gsub(text, " ([_A-Za-z0-9-]+)%.([_A-Za-z0-9-]+)%.(%S+)%s?", Cellular:Link("%1.%2.%3"))
        text = string.gsub(text, " ([_A-Za-z0-9-]+)%.([_A-Za-z0-9-]+)%.(%S+)%:([_0-9-]+)%s?", Cellular:Link("%1.%2.%3:%4"))
    end
    return text
end