local list = { } -- history list of names

function Cellular:LoadOptions()
	self:UpdateHistoryList()
	if self.opts then return end
	
	local db = self.db.profile
	self.dewdrop = AceLibrary("Dewdrop-2.0")
	self.opts = {
		type = "group",
		args = {
			header = {
				type = "header",
				name = "Cellular",
				order = 1,
			},
			backdrop = {
				type = "text",
				name = "Background texture",
				desc = "Background texture",
				set = function(v) db.bg = v self:UpdateLook() end,
				get = function() return db.bg end,
				validate = self.surface:List(),
				order = 2,
			},
			backdropcolor = {
				type = "color",
				name = "Background color",
				desc = "Background color",
				set = function(r,g,b) db.bgr,db.bgg,db.bgb = r,g,b self:UpdateLook() end,
				get = function() return db.bgr, db.bgg, db.bgb end,
				order = 3,
			},
			border = {
				type = "toggle",
				name = "Show border",
				desc = "Show border",
				set = function(v) db.border = v self:UpdateLook() end,
				get = function() return db.border end,
				order = 4,
			},
			bordercolor = {
				type = "color",
				name = "Border color",
				desc = "Border color",
				set = function(r,g,b) db.bordr,db.bordg,db.bordb = r,g,b self:UpdateLook() end,
				get = function() return db.bordr,db.bordg,db.bordb end,
				order = 5,
			},
			inccolor = {
				type = "color",
				name = "Incoming color",
				desc = "Text color for incoming text",
				set = function(r,g,b) db.inr,db.ing,db.inb = r,g,b end, 
				get = function() return db.inr,db.ing,db.inb end, 
				order = 6,
			},
			outcolor = {
				type = "color",
				name = "Outgoing color",
				desc = "Text color for outgoing text",
				set = function(r,g,b) db.outr,db.outg,db.outb = r,g,b end, 
				get = function() return db.outr,db.outg,db.outb end, 
				order = 7,
			},
			fontsize = {
				type = "range",
				name = "Font size",
				desc = "Font size",
				set = function(v) db.fontsize = v self:UpdateLook() end,
				get = function() return db.fontsize end,
				min = 6, max = 30, step = 1,
				order = 8,
			},
			alpha = {
				type = "range",
				name = "Opacity",
				desc = "Opacity",
				set = function(v) db.alpha = v self:UpdateLook() end,
				get = function() return db.alpha end,
				min = 0.05, max = 1, step = 0.01,
				order = 9,
			},
			header3 = {
				type = "header",
				order = 10,
			},
			maxwindows = {
				type = "range",
				name = "Max windows",
				desc = "Maximum number of windows allowed to show at once",
				set = function(v) db.maxwindows = v end,
				get = function() return db.maxwindows end,
				min = 1, max = 12, step = 1,
				order = 11,
			},
			fade = {
				type = "toggle",
				name = "Fade",
				desc = "Enables fading old messages",
				set = function(v) 
					db.fade = v 
					for _,f in ipairs(self.windows) do
						f.msg:SetFading(db.fade)
					end
				end,
				get = function() return db.fade end,
				order = 12,
			},
			busy = {
				type = "text", 
				name = "Busy Message",
				desc = "Set your busy message",
				order = 13,
				set = function(v) db.busymessage = v end,
				get = function() return db.busymessage end,
				usage = "<Your busy message here>",
			},
			history = {
				type = "group",
				name = "History",
				desc = "History",
				order = 14,
				args = {
					disable = {
						type = "toggle",
						name = "Enabled",
						desc = "Toggle history logging",
						set = function(v) 
							db.history = v 
							if not v then 
								self:Print("History logging disabled.  Old data will remain until manually cleared.") 
							end 
						end,
						get = function() return db.history end,
						order = 1,
					},
					show = {
						type = "text", 
						name = "Show",
						desc = "Show",
						order = 2,
						set = function(v) 
							local id = self.usedwindows[v]
							if not id or not self.windows[id]:IsVisible() then 
								self:SetupWindow(v, true) 
							end 
						end,
						get = function() return end,
						validate = self:GetHistoryList(),
					},
					clear = {
						type = "text",
						name = "Clear",
						desc = "Clear",
						order = 3,
						set = function(v) Cellular_History[v] = nil self:UpdateHistoryList() end,
						get = function() return end,
						validate = self:GetHistoryList(),
					},
					clearall = {
						type = "execute",
						name = "Clear All",
						desc = "Clear All",
						order = 4,
						func = function() 
							for name in pairs(Cellular_History) do
								Cellular_History[name] = nil
							end
							self:UpdateHistoryList()
						end,
					},
				},
			},

		},
	}
	
	self:RegisterEvent("Surface_SetGlobal", "UpdateLook")
end

-- update history list
function Cellular:UpdateHistoryList()
	for index in pairs(list) do
		list[index] = nil
	end
	for name in pairs(Cellular_History) do
		table.insert(list, name)
	end
	table.sort(list)
end
function Cellular:GetHistoryList()
	return list
end

-- update the look
function Cellular:UpdateLook()
	local db = self.db.profile
	for i = 1, self.numwindows, 1 do
		local f = self.windows[i]
		f:SetBackdrop({ bgFile = self.surface:Fetch(db.bg), tile = db.bg=="Tooltip", tileSize = 16,
		                 edgeFile = (db.border and "Interface\\Tooltips\\UI-Tooltip-Border") or "", edgeSize = 16,
		                 insets = {left = 4, right = 4, top = 4, bottom = 4}, })
		f:SetBackdropColor(db.bgr, db.bgg, db.bgb)
		f:SetBackdropBorderColor(db.bordr, db.bordg, db.bordb)
		f:SetAlpha(db.alpha)
		f.msg:SetFont(ChatFontNormal:GetFont(), db.fontsize)
	end
end


-- main whisper windows
function Cellular:CreateWindow()
	local db = self.db.profile
	
	if self.numwindows >= db.maxwindows then return end
	self.numwindows = self.numwindows + 1
	
	local num = self.numwindows
	
	-- the main frame, lots to do
	local f = CreateFrame("Button", nil, UIParent)
	f:SetWidth(db.width)
	f:SetHeight(db.height)
	f:ClearAllPoints()
	if db.pos[num] then
		f:SetPoint("TOPLEFT", UIParent, "TOPLEFT", db.pos[num].x, db.pos[num].y)
	else
		db.pos[num] = { }
		f:SetPoint("CENTER", UIParent, "CENTER", (num-1)*20, -(num-1)*20)
	end
	f:SetBackdrop({ bgFile = self.surface:Fetch(db.bg), tile = db.bg=="Tooltip", tileSize = 16,
	                 edgeFile = (db.border and "Interface\\Tooltips\\UI-Tooltip-Border") or "", edgeSize = 16,
	                 insets = {left = 4, right = 4, top = 4, bottom = 4}, })
	f:SetBackdropColor(db.bgr, db.bgg, db.bgb)
	f:SetBackdropBorderColor(db.bordr, db.bordg, db.bordb)
	f.id = num
	f:SetMovable(true)
	f:SetResizable(true)
	f:EnableMouseWheel(true)
	f:RegisterForClicks("LeftButtonUp", "RightButtonUp")
	f:RegisterForDrag("LeftButton")
	f:SetScript("OnDragStart", function() this:StartMoving() end)
	f:SetScript("OnDragStop", function()
		this:StopMovingOrSizing()
		local a,b,c,d,e = this:GetPoint()
		if a == "TOPLEFT" and c == "TOPLEFT" then
			db.pos[this.id] = { x= math.floor(d+0.5), y=math.floor(e+0.5), }
		end
	end)
	f:SetScript("OnClick", function() 
		if arg1 == "LeftButton" then
			if self.attached == this.id then
				ChatFrameEditBox:Hide()
			else
				self:AttachEditBox(this.name, this.id) 
			end
		else
			self:LoadOptions()
			self.dewdrop:Open( this, 
				'children', function() self.dewdrop:FeedAceOptionsTable(self.opts) end, 
				'point', "TOPRIGHT", 'relativePoint', "TOPLEFT" ) 
		end
	end)
	f:SetScript("OnDoubleClick", function() self:MinimizeWindow(true) end)
	f:SetScript("OnHide", function() 
		if self.attached == this.id then 
			self:ResetEditBox() 
		end 
		if this.mini then 
			self:MinimizeWindow(true) 
		end
		self.usedwindows[this.name] = nil
	end)
	f:SetClampedToScreen(true)
	
	-- resizing button
	f:SetMinResize(140, 130)
	f.resizer = CreateFrame("Button", nil, f, "UIPanelButtonGrayTemplate")
	f.resizer:SetHeight(8)
	f.resizer:SetWidth(8)
	f.resizer:SetPoint("BOTTOMRIGHT", f, "BOTTOMRIGHT", -2, 2)
	f.resizer:SetScript("OnMouseDown", function()
		this:GetParent():StartSizing("BOTTOMRIGHT")
		this.sizing = 1
	end)
	f.resizer:SetScript("OnMouseUp", function()
		if this.sizing then
			this:GetParent():StopMovingOrSizing()
			this.sizing = nil
			db.height = math.floor(this:GetParent():GetHeight() + 0.5)
			db.width = math.floor(this:GetParent():GetWidth() + 0.5)
			self:UpdateSizes()
		end
	end)
	
	f.msg = CreateFrame("ScrollingMessageFrame", nil, f)
	f.msg:SetHeight(1)
	f.msg:SetWidth(1)
	f.msg:ClearAllPoints()
	f.msg:SetPoint("TOPLEFT", 24, -18)
	f.msg:SetPoint("BOTTOMRIGHT", -20, 8)
	f.msg:UnregisterAllEvents()
	f.msg:SetFont(ChatFontNormal:GetFont(), db.fontsize)
	f.msg:SetJustifyH("LEFT")
	f.msg:SetTimeVisible(120)
	f.msg:SetMaxLines(100)
	f.msg:SetFading(db.fade)
	f.msg:SetScript("OnHyperlinkClick", function() ChatFrame_OnHyperlinkShow(arg1, arg2, arg3) end)
	f.msg:Show()
	
	-- allow mousewheel scrolling
	f:SetScript("OnMouseWheel", function() 	
		if not arg1 then return end
		if arg1 > 0 then
			self:ScrollUp(this.msg)
			self:ScrollUp(this.msg)
			self:ScrollUp(this.msg)
		elseif arg1 < 0 then
			self:ScrollDown(this.msg)
			self:ScrollDown(this.msg)
			self:ScrollDown(this.msg)
		end 
	end)
	
	-- title text
	f.nametext = f:CreateFontString(nil, "OVERLAY")
	f.nametext:SetFont(GameFontHighlightSmall:GetFont(), 12)
	f.nametext:SetJustifyH("LEFT")
	f.nametext:ClearAllPoints()
	f.nametext:SetPoint("TOPLEFT", f, "TOPLEFT", 8, -7)
	f.nametext:SetPoint("BOTTOMRIGHT", f, "TOPRIGHT", -8, -18)
	
	-- all the buttons
	for b,t in pairs(self.buttons) do
		self:CreateButton(f, b, 16, 16, t.path, t.func, t.p, t.x, t.y, t.tt, t.tpl)
	end
	f.close:SetText("x")
	f.minimize:SetText("-")
	
	f:SetAlpha(db.alpha)
	self.windows[num] = f
	return self.windows[num]
end

-- button creation and layout handler
function Cellular:CreateButton(parent, name, w, h, texture, func, point, x, y, tt, template)
	local self = Cellular
	local button = CreateFrame("Button", nil, parent, template)
	button:SetWidth(w)
	button:SetHeight(h)
	button:ClearAllPoints()
	button:SetPoint(point, parent, point, x, y)
	if texture then
		button:SetNormalTexture(texture)
		button:SetHighlightTexture("Interface\\Minimap\\UI-Minimap-ZoomButton-Highlight")
	end
	button:SetScript("OnMouseDown", function() this:SetAlpha(0.7) end)
	button:SetScript("OnMouseUp", function() this:SetAlpha(1) end)
	button:SetScript("OnClick", function() func() end)
	if tt then
		if tt == 1 then
			button:SetScript("OnEnter", function() 
				GameTooltip:SetOwner(this, "ANCHOR_BOTTOMLEFT")
				GameTooltip:SetText(name) 
				GameTooltip:Show()
			end)
		elseif tt == 2 then
			button:SetScript("OnEnter", function() 
				GameTooltip:SetOwner(this, "ANCHOR_BOTTOMLEFT")
				GameTooltip:SetText(name) 
				GameTooltip:AddLine(" Shift-click to execute")
				GameTooltip:Show()
			end)
		end
		button:SetScript("OnLeave", function() GameTooltip:Hide() end)
	end
	
	parent[name] = button
end

-- button functions
function Cellular:Who()
	SendWho(this:GetParent().name)
end
function Cellular:Busy(name)
	local bm = Cellular.db.profile.busymessage
	if bm and bm ~= "" then
		SendChatMessage(bm, "WHISPER", nil, name or this:GetParent().name)
	end
end
function Cellular:Invite()
	if IsShiftKeyDown() then
		InviteUnit(this:GetParent().name)
	end
end
function Cellular:Friend()
	if IsShiftKeyDown() then
		AddFriend(this:GetParent().name)
	end
end
function Cellular:Ignore()
	if IsShiftKeyDown() then
		AddIgnore(this:GetParent().name)
	end
end
function Cellular:ScrollUp(frame)
	(frame or this:GetParent().msg):ScrollUp()
end
function Cellular:ScrollDown(frame)
	(frame or this:GetParent().msg):ScrollDown()
end
function Cellular:ScrollBottom(frame)
	(frame or this:GetParent().msg):ScrollToBottom()
end
function Cellular:CloseWindow()
	this:GetParent():Hide()
	if this.mini then
		self:MinimizeWindow()
	end
end
function Cellular:MinimizeWindow(doubled)
	local self = Cellular
	local f
	if doubled then
		f = this
	else
		f = this:GetParent()
	end	
	if not f.mini then
		f.mini = 1
		f:SetHeight(30)
		f.msg:Hide()
		for butt in pairs(self.buttons) do
			if butt ~= "close" and butt ~= "minimize" then
				f[butt]:Hide()
			end
		end
		f.resizer:Hide()
		f.mininew = 0
	else
		f.mini = nil
		f:SetHeight(math.max(self.db.profile.height, 50))
		f.msg:Show()
		for butt in pairs(self.buttons) do
			f[butt]:Show()
		end
		f.resizer:Show()
		f.nametext:SetText(f.name)
	end
end
function Cellular:UpdateSizes() 
	local w,h = self.db.profile.width, self.db.profile.height
	for _,f in pairs(self.windows) do
		f:SetWidth(w)
		f:SetHeight(h)
	end
end
